"""JARVIS database models."""
from datetime import datetime

import marshmallow as ma
from umongo import Document, fields

from jarvis_core.db import JARVIS_INST
from jarvis_core.db.models.actions import Ban, Kick, Mute, Unban, Warning
from jarvis_core.db.models.modlog import Action, Modlog, Note
from jarvis_core.db.models.twitter import TwitterAccount, TwitterFollow


class RawField(fields.BaseField, ma.fields.Raw):
    pass


@JARVIS_INST.register
class Autopurge(Document):
    guild = fields.IntegerField(required=True)
    channel = fields.IntegerField(required=True)
    delay = fields.IntegerField(default=30)
    admin = fields.IntegerField(required=True)
    created_at = fields.DateTimeField(default=datetime.now)


@JARVIS_INST.register
class Autoreact(Document):
    guild = fields.IntegerField(required=True)
    channel = fields.IntegerField(required=True)
    reactions = fields.ListField(fields.StringField())
    admin = fields.IntegerField(required=True)
    created_at = fields.DateTimeField(default=datetime.now)


@JARVIS_INST.register
class Config(Document):
    """Config database object."""

    key = fields.StringField(required=True)
    value = RawField(required=True)


@JARVIS_INST.register
class Guess(Document):
    """Guess database object."""

    correct = fields.BooleanField(default=False)
    guess = fields.StringField(required=True)
    user = fields.IntegerField(required=True)


@JARVIS_INST.register
class Lock(Document):
    """Lock database object."""

    active = fields.BooleanField(default=True)
    admin = fields.IntegerField(required=True)
    channel = fields.IntegerField(required=True)
    duration = fields.IntegerField(default=10)
    guild = fields.IntegerField(required=True)
    reason = fields.StringField(required=True)
    created_at = fields.DateTimeField(default=datetime.utcnow)


@JARVIS_INST.register
class Purge(Document):
    """Purge database object."""

    admin = fields.IntegerField(required=True)
    channel = fields.IntegerField(required=True)
    guild = fields.IntegerField(required=True)
    count = fields.IntegerField(default=10)
    created_at = fields.DateTimeField(default=datetime.utcnow)


@JARVIS_INST.register
class Reminder(Document):
    """Reminder database object."""

    active = fields.BooleanField(default=True)
    user = fields.IntegerField(required=True)
    guild = fields.IntegerField(required=True)
    channel = fields.IntegerField(required=True)
    message = fields.StringField(required=True)
    remind_at = fields.DateTimeField(required=True)
    created_at = fields.DateTimeField(default=datetime.utcnow)
    private = fields.BooleanField(default=False)


@JARVIS_INST.register
class Rolegiver(Document):
    """Rolegiver database object."""

    guild = fields.IntegerField(required=True)
    roles = fields.ListField(fields.IntegerField())


@JARVIS_INST.register
class Roleping(Document):
    """Roleping database object."""

    active = fields.BooleanField(default=True)
    role = fields.IntegerField(required=True)
    guild = fields.IntegerField(required=True)
    admin = fields.IntegerField(required=True)
    bypass = fields.DictField()
    created_at = fields.DateTimeField(default=datetime.utcnow)


@JARVIS_INST.register
class Setting(Document):
    """Setting database object."""

    guild = fields.IntegerField(required=True)
    setting = fields.StringField(required=True)
    value = RawField()


@JARVIS_INST.register
class Star(Document):
    """Star database object."""

    active = fields.BooleanField(default=True)
    index = fields.IntegerField(required=True)
    message = fields.IntegerField(required=True)
    channel = fields.IntegerField(required=True)
    starboard = fields.IntegerField(required=True)
    guild = fields.IntegerField(required=True)
    admin = fields.IntegerField(required=True)
    star = fields.IntegerField(required=True)
    created_at = fields.DateTimeField(default=datetime.utcnow)


@JARVIS_INST.register
class Starboard(Document):
    """Starboard database object."""

    channel = fields.IntegerField(required=True)
    guild = fields.IntegerField(required=True)
    admin = fields.IntegerField(required=True)
    created_at = fields.DateTimeField(default=datetime.utcnow)
